# -*- coding: utf-8 -*-

import os


#------------------------------------------------------------------------------
# xbmc related
#------------------------------------------------------------------------------
import xbmc, xbmcaddon

__settings__ = xbmcaddon.Addon(id='plugin.video.RoiD')
__icon__ = xbmcaddon.Addon(id='plugin.video.RoiD').getAddonInfo('icon')
translate = __settings__.getLocalizedString
enable_debug = True
language = xbmc.getLanguage

def log(msg, level=xbmc.LOGDEBUG):
	plugin = "RoiD"
	msg = msg.encode('utf-8')

	xbmc.log("[%s] %s" % (plugin, msg.__str__()), level)

def getSetting(name):
	return __settings__.getSetting(name)

def setSetting(name, value):
	__settings__.setSetting(id=name, value=value)

def showNotification(title, message, timeout=2000, icon=__icon__):
	def clean(s):
		return str(s.encode('utf-8', 'ignore'))
	command = ''
	if icon:
		command = 'Notification(%s,%s,%s,%s)' % (clean(title), clean(message), timeout, icon)
	else:
		command = 'Notification(%s,%s,%s)' % (clean(title), clean(message), timeout)
	xbmc.executebuiltin(command)

def runPlugin(url):
	xbmc.executebuiltin('XBMC.RunPlugin(' + url +')')

#------------------------------------------------------------------------------
# dialogs
#------------------------------------------------------------------------------
from dialogs.dialogQuestion import DialogQuestion
from dialogs.dialogBrowser import DialogBrowser
from dialogs.dialogInfo import DialogInfo
from dialogs.dialogError import DialogError
from dialogs.dialogKeyboard import GetKeyboardText

from utils.xbmcUtils import getKeyboard

def ask(question):
	diaQuestion = DialogQuestion()
	return diaQuestion.ask(question)

def showInfo(message):
	diaInfo = DialogInfo()
	diaInfo.show(message)   

def showError(message):
	diaError = DialogError()
	diaError.show(message)

def browseFolders(head):
	diaFolder = DialogBrowser()
	return diaFolder.browseFolders(head)

def showOSK(defaultText='', title='', hidden=False):
	return getKeyboard(defaultText, title, hidden)

#------------------------------------------------------------------------------
# Net
#------------------------------------------------------------------------------
from net import Net
# Net
addon_id = xbmcaddon.Addon().getAddonInfo('id')
addon_profile_path = xbmc.translatePath(os.path.join(xbmc.translatePath('special://profile'), 'addon_data', addon_id))
if not os.path.exists(addon_profile_path):
	try: xbmcvfs.mkdirs(addon_profile_path)
	except: os.mkdir(addon_profile_path)
_cookie_file = xbmc.translatePath(os.path.join(xbmc.translatePath(addon_profile_path), 'cookies.txt'))

def _is_cookie_file(the_file):
	exists = os.path.exists(the_file)
	if not exists:
		return False
	else:
		try:
			tmp = xbmcvfs.File(the_file).read()
			if tmp.startswith('#LWP-Cookies-2.0'):
				return True
			return False
		except:
			with open(the_file, 'r') as f:
				tmp = f.readline()
				if tmp == '#LWP-Cookies-2.0\n':
					return True
				return False


def _create_cookie(the_file):
	try:
		if xbmcvfs.exists(the_file):
			xbmcvfs.delete(the_file)
		_file = xbmcvfs.File(the_file, 'w')
		_file.write('#LWP-Cookies-2.0\n')
		_file.close()
		return the_file
	except:
		try:
			_file = open(the_file, 'w')
			_file.write('#LWP-Cookies-2.0\n')
			_file.close()
			return the_file
		except:
			return ''

if not _is_cookie_file(_cookie_file):
	_cookie_file = _create_cookie(_cookie_file)

net = Net(_cookie_file, cloudflare=True)
	

#------------------------------------------------------------------------------
# web related
#------------------------------------------------------------------------------
from utils.regexUtils import parseTextToGroups
from utils.webUtils import CachedWebRequest
import cookielib
import urllib2
import requests

def getHTML(url, form_data='', referer='', xml=False, mobile=False, ignoreCache=False, demystify=False):
	if url == 'http://www.streamlive.to':
			url = xbmc.translatePath(os.path.join(Paths.filesDir, 'live.xml'))
	if url == 'http://www.tvone1.tv':
			url = xbmc.translatePath(os.path.join(Paths.filesDir, 'tvone.xml'))
	if '[CLOUDFLARE]' in url:
		url = url.replace('[CLOUDFLARE]','').encode('utf-8')
		return net.http_GET(url).content
	if '[REFERER]' in url:
		spl = url.split('[REFERER]')
		url = spl[0]
		referer = spl[1]
	if '[UserInput]' in url:
		TEXTDIALOG = GetKeyboardText('', '')
		url = url.replace('[UserInput]', TEXTDIALOG.decode('utf-8'))
	if '[POST]' in url:
		url = url.replace('/[POST]','[POST]')
		spl = url.split('[POST]')
		url = spl[0]
		tmp_data = spl[1].split('&')
		dat = {}
		for val in tmp_data:
			key = val.split('=',1)[0]
			value = val.split('=',1)[1]
			dat[key]=value
		return requests.Session().post(url, data=dat, verify=False).text
	if '[URL]' in url:
		urls = url.split('[URL]')
		combined = ''
		pg = 0
		for u in urls:
			pg = pg + 1
			combined = combined+"#######################################################################\nRoiDpage"+str(pg)+"="+u+"\n#######################################################################\n\n"+requests.get(u, None).text+"\n\n\n\n\n"
		return combined
	if url.startswith('https://'):
		hdr={}
		if '[HEADERS]' in url:
			spl = url.split('[HEADERS]')
			url = spl[0]
			tmp_data = spl[1].split('&')
			for val in tmp_data:
				key = val.split('=',1)[0]
				value = val.split('=',1)[1]
				hdr[key]=value
		return requests.Session().get(url, headers=hdr, verify=False).text

	cookiePath = xbmc.translatePath(os.path.join(Paths.cacheDir, 'cookies.lwp'))
	request = CachedWebRequest(cookiePath, Paths.cacheDir)
	return request.getSource(url, form_data, referer, xml, mobile, ignoreCache, demystify)

def getCookies(cookieName, domain):
	cookiePath = xbmc.translatePath(os.path.join(Paths.cacheDir, 'cookies.lwp'))
	
	def load_cookies_from_lwp(filename):
		lwp_cookiejar = cookielib.LWPCookieJar()
		lwp_cookiejar.load(filename, ignore_discard=True)
		return lwp_cookiejar
	
	for cookie in load_cookies_from_lwp(cookiePath):
		if domain in cookie.domain and cookieName in cookie.name:
			return cookie.value

def parseWebsite(source, regex, referer='', variables=[]):
	def parseWebsiteToGroups(url, regex, referer=''):
		data = getHTML(url, None, referer)
		return parseTextToGroups(data, regex)

	groups = parseWebsiteToGroups(source, regex, referer)

	if variables == []:
		if groups:
			return groups[0]
		else:
			return ''
	else:
		resultArr = {}
		i = 0
		for v in variables:
			if groups:
				resultArr[v] = groups[i]
			else:
				resultArr[v] = ''
			i += 1
		return resultArr





#------------------------------------------------------------------------------
# classes with constants
#------------------------------------------------------------------------------
class Paths:
	rootDir = xbmc.translatePath(__settings__.getAddonInfo('path')).decode('utf-8')

	resDir = os.path.join(rootDir, 'resources')
	imgDir = os.path.join(resDir, 'images')
	filesDir = os.path.join(resDir, 'files')
	modulesDir = os.path.join(resDir, 'modules')
	catchersDir = os.path.join(resDir,'catchers')
	dictsDir = os.path.join(resDir,'dictionaries')

	pluginFanart = os.path.join(rootDir, 'fanart.jpg')
	defaultVideoIcon = os.path.join(imgDir, 'video.png')
	defaultCategoryIcon = os.path.join(imgDir, 'folder.png')	

	pluginDataDir = xbmc.translatePath(__settings__.getAddonInfo('profile')).decode('utf-8')
	cacheDir = os.path.join(pluginDataDir, 'cache')
	favouritesFolder = os.path.join(pluginDataDir, 'favourites')
	favouritesFile = os.path.join(favouritesFolder, 'favourites.cfg')
	customModulesDir = os.path.join(pluginDataDir, 'custom')
	customModulesFile = os.path.join(customModulesDir, 'custom.cfg')
	
	catchersRepo = ''
	modulesRepo = ''
	customModulesRepo = ''
	
	xbmcFavouritesFile = xbmc.translatePath( 'special://profile/favourites.xml' )
